\name{customSurv}
\alias{customSurv}
\alias{customSurv,matrix,Surv-method}
\alias{customSurv,data.frame,Surv-method}
\alias{customSurv,ExpressionSet,character-method}
\alias{customSurv,ExpressionSet,Surv-method}
\title{Integrate a custom survival model into the \code{survHD} framework}
\description{This function is a wrapper for user-defined survival models which shall be integrated into the \code{survHD}-framework.}
\usage{
\S4method{customSurv}{data.frame,Surv}(X, y, learnind, customSurvModel, essential=FALSE,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{Gene expression data. The following formats are available:
  \itemize{
   \item{\code{matrix}} Rows correspond to observations, columns to variables.
   \item{\code{data.frame}}  Rows correspond to observations, columns to variables.
   \item{\code{ExpressionSet}} survHD will extract the expressions using exprs().
  } }
  \item{y}{Survival Response, an object of class:
  \itemize{
   \item{\code{Surv}} if \code{X} is of type data.frame or matrix
   \item{\code{character}} if \code{X} is of type \code{ExpressionSet}. In
   this case \code{y} may be the name of 
   the survival response in the phenoData of \code{X}. If survival time and indicator are stored separately in the phenoData
   one can specify a two-element character vector the first element representing the survival time variable.
          }}
  \item{learnind}{An index vector specifying the observations that
                  belong to the learning set. May be \code{missing};
                  in that case, the learning set consists of all
                  observations and predictions are made on the
                  learning set.}
  \item{customSurvModel}{A user-defined \code{function} receiving a \code{data.frame Xlearn}, a \code{Surv-object Ylearn}
  and the arguments specified in \dots which fits a survival model and returns an object of class \code{customsurvhd}}
  \item{essential}{shall the model output (see slot model of class \code{\link{ModelLearned}})
                  include the bare essentials only (especially for tune this option might save a lot of memory)?}
  \item{\dots}{Further arguments that shall be passed to function \code{customSurvModel}.}
}
\value{An object of class \code{\link{ModelLearned}} embedding a model-slot of class \code{\link{ModelCustom}}.}
\references{   Binder, H., et al. (2009) Boosting for high-dimensional time-to-event data with
competing risks, Bioinformatics, 25 (7), 890-896
}
\author{Christoph Bernau \email{bernau@ibe.med.uni-muenchen.de}}
\seealso{\code{\link{learnSurvival}},\code{\link{ModelCustom}},\code{\link{predictsurvhd}}} 
\examples{
\dontrun{
	####coxboost as a custom survival model function
	##Xlearn and Ylearn are obligatory inputs 
	customSurvModel<-function(Xlearn,Ylearn,unpenalizedvars=NULL,learnind,...){
		###load required packages
		require(CoxBoost)
		###handle inputs
		ll <- list(...)
		ll$time <- Ylearn[, 1]
		ll$status <- Ylearn[, 2]
		ll$trace = FALSE
		ll$standardize = TRUE
		
		##handle unpenalized var
		if (is.null(unpenalizedvars) == TRUE) Xlearn <- data.frame(Xlearn[, ])
		else Xlearn <- data.frame(unpenalizedvars[learnind, ], Xlearn[, ])
		if (!is.null(unpenalizedvars)) 
			ll$unpen.index <- unpeninds
		
		ll$x <- as.matrix(Xlearn)
		ll$learnind<-NULL
		##call actual model function CoxBoost
		output.coxboost <- do.call("CoxBoost", args = ll)
		
		##define prediction function which will be stored in slot predfun
		##and called by predictsurvhd (signature(ModelCustom))
		###
		predfun<-function(object,newdata,type,timegrid=NULL,...){
			require(CoxBoost)
#either type lp or type SurvivalProbs must be implemented
#for typ lp the obligatory return class is LinearPrediction
			if (type == "lp") {
				survobj <- mod(object)
				pred <- predict(object = survobj, type = "lp", newdata = newdata)
				pred <- structure(pred[1, ], .Names = colnames(pred))
				pred <- new("LinearPrediction", lp = pred)
			}
#for typ SurvivalProbs the obligatory return class is Breslow
			else if (type == "SurvivalProbs") {
				survobj <- mod(object)
				if (is.null(timegrid)) {
					stop("No timegrid specified.")
				}
				curves <- predict(object = survobj, newdata = newdata, times = timegrid, 
						type = "risk")
				pred <- new("breslow", curves = curves, time = timegrid)
				pred <- new("SurvivalProbs", SurvivalProbs = pred)
			}
			
			else stop('Invalid "type" argument.')
			return(pred)  
		}
		
		
		###now create customsurvhd-object (which is the obligatory output-object)
		custommod<-new("ModelCustom",mod=output.coxboost,predfun=predfun,extraData=list())
		
		return(custommod)
	}
	
	###test it
	##data
	data(beer.exprs)
	set.seed(123)
	X<-t(as.matrix(beer.exprs))
	data(beer.survival)
	y<-Surv(beer.survival[,2],beer.survival[,1])
	##learningset	
	ls<-generateLearningsets(y=y[,1],method='CV',fold=3,niter=2)
	
#gene selection
	gsel<-geneSelection(X=X,y=y,method='fastCox',LearningSets=ls,criterion='coefficient')
	##learnsurvival	
	
	svaggr<-learnSurvival(X=X,y=y,GeneSel=gsel,nbgene=100,survmethod='customSurv',customSurvModel=customSurvModel,LearningSets=ls)
	
	###tune
	tuneres<-tune(X=X,y=y,GeneSel=gsel,nbgene=100,survmethod='customSurv',customSurvModel=customSurvModel,LearningSets=ls,grids = list(stepno = c(10 * (1:10))))
	
	###use tuneres in learnSurvival
	svaggr<-learnSurvival(X=X,y=y,GeneSel=gsel,nbgene=100,survmethod='customSurv',customSurvModel=customSurvModel,LearningSets=ls,tuneres=tuneres)
	evaluate(svaggr,measure='CvPLogL')
	evaluate(svaggr,measure='PErrC',timegrid=30:35,gbm=TRUE)
	evaluate(svaggr,measure='PErrC',timegrid=30:35,gbm=FALSE)
	
}
}
\keyword{multivariate}
