\name{generateLearningsets}
\alias{generateLearningsets}
\title{Repeated Divisions into learn- and tets sets}
\description{
  Due to very small sample sizes, the classical division learnset/testset
  does not give accurate information about the prediction performance.
  Therefore, several different divisions should be used and aggregated.
  The implemented methods are discussed in Braga-Neto and Dougherty (2003)
  and Molinaro et al. (2005) whose terminology is adopted.

  This function is usually the basis for all deeper analyses.
}
\usage{
generateLearningsets(n, y, method = c("LOOCV", "CV", "MCCV", "bootstrap"),
                     fold = NULL, niter = NULL, ntrain = NULL, strat = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{The total number of observations in the available data set. May be \code{missing}
           if \code{y} is provided instead.}
  \item{y}{A \code{Surv} object (the survival response).
            \emph{Must} be given if \code{n} is not specified.}
  \item{method}{Which kind of scheme should be used to generate divisions
                into learning sets and test sets ? Can be one of the following:
                 \describe{
                 \item{"LOOCV"}{Leaving-One-Out Cross Validation.}
                 \item{"CV"}{(Ordinary) Cross-Validation. Note that \code{fold}
                             must as well be specified.}
                 \item{"MCCV"}{Monte-Carlo Cross Validation, i.e.
                               random divisions into learning sets with
                               \code{ntrain}(s.below) observations
                               and tests sets with \code{ntrain} observations.}
                 \item{"bootstrap"}{Learning sets are generated
                                    by drawing \code{n} times with replacement from
                                    all observations. Those not drawn not all
                                    form the test set.}
                }
                }
  \item{fold}{Gives the number of CV-groups. Used only when \code{method="CV"}}
  \item{niter}{Number of iterations (s.\code{details).}}
  \item{ntrain}{Number of observations in the learning sets. Used
                only when \code{method="MCCV"}.}	
                \item{strat}{use stratified resampling (w.r.t. censoring)}	
}

\details{
\itemize{
  \item When \code{method="CV"}, \code{niter} gives the number of times
  the whole CV-procedure is repeated. The output matrix has then \code{fold}x\code{niter} rows.
  When \code{method="MCCV"} or \code{method="bootstrap"}, \code{niter} is simply the number of considered
  learning sets.
\item Note that \code{method="CV",fold=n} is equivalent to \code{method="LOOCV"}.
}
}

\value{An object of class \code{\link{LearningSets}}}

\author{

Christoph Bernau \email{bernau@ibe.med.uni-muenchen.de}
Martin Slawski \email{ms@cs.uni-sb.de}
Anne-Laure Boulesteix \email{boulesteix@ibe.med.uni-muenchen.de}
        }

\seealso{\code{\link{LearningSets}}, \code{\link{geneSelection}}, \code{\link{tune}},
         \code{\link{learnSurvival}}}
\examples{
	set.seed(2)
	nsamples <- 100
	X <- matrix(rnorm(nsamples*200),nrow=nsamples)
	colnames(X) <- make.names(1:ncol(X))
	rownames(X) <- make.names(1:nrow(X))
	time <- rexp(nsamples)
	cens <- sample(0:1,size=nsamples,replace=TRUE)
	y <- Surv(time,cens)
        ##stratified cross validation 
	LearningSets_cv <- generateLearningsets(y = y, method = "CV", fold = 5, niter = 1, strat = TRUE)
        ##stratified bootstrap
        LearningSets_boot<- generateLearningsets(y = y, method = "bootstrap", niter = 100, strat = TRUE)
}

\keyword{multivariate}
