\name{learnSurvival}
\alias{learnSurvival}
\alias{learnSurvival,ExpressionSet,character-method}
\alias{learnSurvival,ExpressionSet,Surv-method}
\alias{learnSurvival,data.frame,Surv-method}
\alias{learnSurvival,matrix,Surv-method}
\title{General method for survival analysis with various methods}
\description{
  Most general function in the package, providing an interface
  to perform variable selection, hyperparameter tuning and
  survival in one step. Alternatively, the first two
  steps can be performed separately and can then be plugged into
  this function.\cr
  For S4 method information, s. \code{\link{learnSurvival-methods}}.
}
\usage{
\S4method{learnSurvival}{data.frame,Surv}(X, y, LearningSets, GeneSel, GeneSellist = list(), nbgene, survmethod, tuneres, tuninglist = list(), trace = TRUE, innercv=FALSE, addtune=list(), threshold=NULL,...)
}
\arguments{
 \item{X}{Gene expression data. The following formats are available:
  \itemize{
   \item{\code{matrix}} Rows correspond to observations, columns to variables.
   \item{\code{data.frame}}  Rows correspond to observations, columns to variables.
   \item{\code{ExpressionSet}} survHD will extract the expressions using exprs().
  } }
  \item{y}{Survival Response, an object of class:
  \itemize{
   \item{\code{Surv}} if \code{X} is of type data.frame or matrix
   \item{\code{character}} if \code{X} is of type \code{ExpressionSet}. In
   this case \code{y} may be the name of 
   the survival response in the phenoData of \code{X}. If survival time and indicator are stored separately in the phenoData
   one can specify a two-element character vector the first element representing the survival time variable.
          }}
  
  \item{LearningSets}{An object of class \code{\link{LearningSets}}. May
                      be missing, then the complete datasets is used as
                      learning set.}
  \item{GeneSel}{Optional (but usually recommended) object of class
                 \code{\link{GeneSel}} containing variable importance
                 information for the argument \code{LearningSets}}
  \item{GeneSellist}{In the case that the argument \code{GeneSel} is missing,
                     this is an argument list passed to \code{\link{geneSelection}}.
                     If both \code{GeneSel} and \code{GeneSellist} are missing,
                     no variable selection is performed.}
  \item{nbgene}{Number of best genes to be kept for survival analysis, based
                on either \code{GeneSel} or the call to \code{\link{geneSelection}}
                using \code{GeneSellist}. In the case that both are missing,
                this argument is not necessary.
		}
  \item{survmethod}{Name of the function (as character) ending with \code{surv} indicating
                    the survival method to be used.}
  \item{tuneres}{Analogous to the argument \code{GeneSel} - object of
                 class \code{\link{TuneOut}} containing information
                 about the best hyperparameter choice for the argument
                 \code{LearningSets}.}
  \item{tuninglist}{Analogous to the argument \code{GeneSellist}. In the
                    case that the argument \code{tuneres} is missing, this
                    argument list is passed to \code{\link{tune}}.
                    If both \code{tuneres} and \code{tuninglist} are missing,
                    no variable selection is performed.
                    \bold{warning}: Note that if a user-defined hyperparameter
                    grid is passed, this will result in a list within a list:
                    \code{tuninglist = list(grids=list(argname = c())}, s. example.
                    \bold{warning}: Contrary to \code{\link{tune}}, if
                    \code{tuninglist} is an empty list (default),
                    \emph{no} hyperparameter tuning
                    will be performed at all. To use pre-defined hyperparameter
                    grids, the argument is \code{tuninglist = list(grids = list())}.}
  \item{trace}{Should progress be traced ? Default is \code{TRUE}.}
  \item{innercv}{logical indicating whether learnSurvival is used in an inner loop (X matrix will not be stored)}
  \item{addtune}{list with additional parameters for evaluating tuneres (these arguments are passed to method \code{evaluationSurv})}
  \item{threshold}{threshold for \code{pvalue} or \code{coefficient} when selecting genes}
  \item{\dots}{Further arguments passed to the appropriate function
  corresponding to the \code{survmethod} argument.}
}
\details{For details about hyperparameter tuning, consult \code{\link{tune}}.}
\value{An object of class \code{LearnOut}}
\references{
Verweij, P.J.M. and van Houweilingen, H.C. (1993) Cross-Validation in Survival Analysis, Statistics in Medicine, 12, 2305-2314

Porzelius, Ch., et al. (2011) The benefit of data-based model complexity selection via prediction error curves in time-to-event data, Computational Statistics, 26, 293-302
}
\author{
    Christoph Bernau \email{bernau@ibe.med.uni-muenchen.de}
    Levi Waldron \email{lwaldron@hsph.harvard.edu}
}
\seealso{\code{\link{geneSelection}}, \code{\link{tune}}, \code{\link{evaluate}},
         \code{\link{coxBoostSurv}}, \code{\link{plusMinusSurv}}}
\examples{
set.seed(2)
nsamples <- 100
X <- matrix(rnorm(nsamples*200),nrow=nsamples)
colnames(X) <- make.names(1:ncol(X))
rownames(X) <- make.names(1:nrow(X))
time <- rexp(nsamples)
cens <- sample(0:1,size=nsamples,replace=TRUE)
y <- Surv(time,cens)
##LearningSets define outer fold:
LearningSets <- generateLearningsets(y = y, method = "CV", fold = 5, niter = 1, strat = TRUE)
###fit model on each fold
cval <- learnSurvival(X = X, y = y, LearningSets = LearningSets,
        survmethod="coxBoostSurv",
        stepno=40)
}
\keyword{multivariate}
