\name{Prediction from LearnOut objects}
\docType{methods}
\alias{predict,LearnOut-method}
\alias{predictsurvhd,LearnOut,data.frame-method}
\alias{predictsurvhd,LearnOut,missing-method}
\alias{predictsurvhd,ModelLearned,missing-method}
\title{Prediction from survival models in a LearnOut object}
\description{Predictions from (possilbly several) survival models which are aggregated in a LearnOut object.}
\usage{
\S4method{predictsurvhd}{LearnOut,data.frame}(object, newdata,type="lp",timegrid=NULL,gbm=TRUE,indices=NULL, addtrain=FALSE,voting_scheme=NULL,...)
\S4method{predict}{LearnOut}(object,...)
}
\arguments{

  \item{object}{object of class \code{LearnOut}}
  \item{newdata}{Gene expression data for which predictions shall be returned. The following formats are available:
  \itemize{
   \item{\code{matrix}} Rows correspond to observations, columns to variables.
   \item{\code{data.frame}}  Rows correspond to observations, columns to variables.
   \item{\code{ExpressionSet}} survHD will extract the expressions using exprs().
  } }
  \item{type}{class \code{character}, can be one of the following:
  \itemize{
  \item "lp", if the linear predictor is to be computed
  \item "SurvivalProbs", if the survival probabilities are to be computed
  \item "cp", if a classification into several risk groups shall be performed 
  }}
  \item{timegrid}{the timepoints for which the survival probabilities are to be computed}
  \item{gbm}{shall the default estimation procedure from package \code{gbm} be used for estimating the basehazard?}
  \item{indices}{specifies the models in slot \code{ModelLearnedlist} of the \code{LearnOut} object for which predictions shall be returned}
  \item{addtrain}{defaults to \code{FALSE} which means that predictions are returned for test data only. If set to \code{TRUE} predictions are returned for training and test data.}
  \item{voting_scheme}{for type "cp" \code{ppredict} usually returns a matrix containing the risk categories for different models. If a voting_scheme is specified \code{predict} returns 
  a single predicted category for each observation which is obtained by applying the specified \code{voting scheme} for aggregating the different model classifications to a single one. This argument can e one 
  of the following
  
  \itemize{
  \item "first": majortiy vote handling ties by using the first appearance of the maximum number of votes 
   \item "last": majortiy vote handling ties by using the last appearance of the maximum number of votes 
   \item "random": majortiy vote handling ties by selecting randomly among ties
   \item an object of class "function": uses the specified function for performing the voting. The function has to accept a character vector (containing a patient's different
   risk classifcations from the different models) and has to return a character vector of length one (s.section \code{examples}).
  }
  
  
  }
  \item{\dots}{currently unused argument}
}


\author{
Christoph Bernau \email{bernau@ibe.med.uni-muenchen.de}
Levi Waldron \email{lwaldron@hsph.harvard.edu}
 Markus Riester <riester@jimmy.harvard.edu>
}

\examples{
\dontrun{
##prepare data
	data(beer.exprs)
	set.seed(123)
	X<-t(as.matrix(beer.exprs))
	data(beer.survival)
	y<-Surv(beer.survival[,2],beer.survival[,1])
	
	ls<-generateLearningsets(y=y[,1],method='CV',fold=3,niter=2)
	
	svaggr<-learnSurvival(X=X,y=y,survmethod='coxBoostSurv',LearningSets=ls)
	
	##predict linear predictors for the test observations in each cv-step 
	pp1<-predict(svaggr)
	##predict survival probabilities for the test observations in each cv-step 
	pp2<-predict(svaggr,type='SurvivalProbs',time=1:3)
	##classify test observations in each cv-step according to their predicted lps 
	pp3<-predict(svaggr,type='cp')
	##predict lps for training and test observations in each cv-step 
	pp4<-predict(svaggr,addtrain=T)
	##predict survival probabilities for training and test observations in each cv-step 
	pp5<-predict(svaggr,type='SurvivalProbs',time=1:3,addtrain=T)
	
	##predict lps for newdata based on the different models of the cv
	pp6<-predict(svaggr,newdata=X)
	##predict lps for newdata and original observations based on the different models of the cv
	pp7<-predict(svaggr,newdata=X,addtrain=T)
	##predict SurvivalProbs for newdata based on the different models of the cv
	pp8<-predict(svaggr,newdata=X,type='SurvivalProbs',time=1:3)
	##predict SurvivalProbs for newdata and original observations based on the different models of the cv
	pp9<-predict(svaggr,newdata=X,type='SurvivalProbs',time=1:3,addtrain=T)
	##classify newdata according to the lps predicted by the different mdels in the cv
	pp10<-predict(svaggr,newdata=X,type='cp')
	
	###use voting scheme
	###if a voting scheme is specified predict returns a single classification for each observation (usually the majority vote of the different models in ModelLearnedlist of the LearnOut-object)
	
	##return majority vote; for ties the first category which has the maximum number of votes 
	pp11<-predict(svaggr,type='cp',voting_scheme='first')
	##return majority vote; for ties, select ranomly among categories with the maximum number of votes
	pp11<-predict(svaggr,type='cp',voting_scheme='first')
	##use user-specified voting mechanism (corresponds to 'random' here)
	voting_scheme<-function(x){names(table(x))[sample(which(table(x)==max(table(x))),1)]}
	pp12<-predict(svaggr,type='cp',voting_scheme=voting_scheme)
	
	

}

}
\seealso{\code{\link{gbmbasehaz}}}

\keyword{multivariate}