\name{tune}
\alias{tune}
\alias{tune,matrix,Surv-method}
\alias{tune,ExpressionSet,character-method}
\alias{tune,ExpressionSet,Surv-method}
\alias{tune,data.frame,Surv-method}
\title{Hyperparameter tuning for survival methods}
\description{Most survival methods implemented in this package depend on one
or even several hyperparameters (s. details) that should be optimized
to obtain good (and comparable !) results. As tuning scheme, we propose
three fold Cross-Validation on each \code{learningset} (for fixed selected
variables). Note that \code{LearningSets} usually do not contain the
complete dataset, so tuning involves a second level of splitting the dataset.
Increasing the number of folds leads to larger datasets (and possibly to higher accuracy),
but also to higher computing times.\cr
For S4 method information, s. \code{link{tune-methods}}}
\usage{
\S4method{tune}{data.frame,Surv}(X, y, LearningSets, measure, GeneSel, GeneSellist = list(), nbgene, survmethod, fold = 3, strat = FALSE, grids = list(), trace = TRUE, threshold=NULL, ...)
}
\arguments{
 \item{X}{Gene expression data. The following formats are available:
  \itemize{
   \item{\code{matrix}} Rows correspond to observations, columns to variables.
   \item{\code{data.frame}}  Rows correspond to observations, columns to variables.
   \item{\code{ExpressionSet}} survHD will extract the expressions using exprs().
  } }
  \item{y}{Survival Response, an object of class:
  \itemize{
   \item{\code{Surv}} if \code{X} is of type data.frame or matrix
   \item{\code{character}} if \code{X} is of type \code{ExpressionSet}. In this case y is the name of 
   the survival response in the phenoData of \code{X}. If survival time and indicator are stored separately in the phenoData
   one can specify a two-element character vector the first element representing the survival time variable.
          }}
              
  \item{LearningSets}{An object of class \code{\link{LearningSets}}. May
                      be missing, then the complete datasets is used as
                      learning set.}
 \item{measure}{the evaluation criterion used when determining the best tuning parameter}     
  \item{GeneSel}{Optional (but for certain methods recommended) object of class
                 \code{\link{GeneSel}} containing variable importance
                 information for the argument \code{LearningSets}}
  \item{GeneSellist}{In the case that the argument \code{GeneSel} is missing,
                     this is an argument list passed to \code{\link{geneSelection}}.
                     If both \code{GeneSel} and \code{GeneSellist} are missing,
                     no variable selection is performed.}
  \item{nbgene}{Number of best genes to be kept for survival analysis, based
                on either \code{GeneSel} or the call to \code{\link{geneSelection}}
                using \code{GeneSellist}. In the case that both are missing,
                this argument is not necessary.
                \bold{note}:
		\itemize{
  
                \item If the gene selection method has been one of
                      \code{"lasso", "elasticnet", "boosting"},
                      \code{nbgene} will be reset to \code{min(s, nbgene)}
                      where \code{s} is the number of nonzero coefficients.
                
		    }
		}
  \item{survmethod}{Name of function ending with \code{surv} indicating
                    the survival method to be used.}
  \item{fold}{The number of cross-validation folds used within each \code{learningset}.
              Default is 3. Increasing \code{fold} will lead to higher computing times.}
                \item{strat}{use stratified resampling (w.r.t. censoring) in the inner cv loop}	
  \item{grids}{A named list. The names correspond to the arguments to be tuned,
               e.g. \code{stepno} (the number of steps) for \code{\link{coxBoostSurv}}. Each element 
               is a \code{numeric}
               vector defining the grid of candidate values. Of course, several hyperparameters
               can be tuned simultaneously (though requiring much time). By
               default, \code{grids} is an empty list. In that case, a pre-defined
               list will be used, s. details.}
  \item{trace}{Should progress be traced ? Default is \code{TRUE}.}
  \item{threshold}{threshold for \code{pvalue} or \code{coefficient} when selecting genes}
  \item{\dots}{Further arguments to be passed to \code{survmethod},
               not one of the arguments to be tuned.}
}
\note{The computation time can be enormously high. Note that for each different
      \code{learningset}, the classifier must be trained \code{fold} times
      \code{number of possible different hyperparameter combinations} times.
      E.g. if the number of the LearningSets is fifty, \code{fold = 3} and
      two hyperparameters (each with 5 candidate values) are tuned, 50x3x25=3750
      training iterations are necessary !}
\details{If no \code{measure} is specified, survHD will fit all the models 
needed for the inner cross validation loop without evaluating them. The models will be automatically 
evaluated in the function \code{getBestParameters} which is usually called in the subsequent call to \code{learnSurvival}.
This way one can use different meaures as tuning criterion without refitting the models of the inner cross-validation loop.
However, the storage of many models in the tuning result object can be very memory-intensive.  


The following default settings are used, if the arguments \code{grids}
         is an empty list:
         \describe{
         
         \item{\code{\link{coxBoostSurv}}}{\code{stepno=c(10*(1:10))}}
         }
}
\value{An object of class \code{\link{TuneOut}}}

\references{
Porzelius, Ch., et al. (2011) The benefit of data-based model complexity selection via prediction error curves in time-to-event data, Computational Statistics, 26, 293-302
}

\author{Christoph Bernau \email{bernau@ibe.med.uni-muenchen.de}

Martin Slawski \email{ms@cs.uni-sb.de}
        }

\seealso{\code{\link{TuneOut}}, \code{\link{geneSelection}}, \code{\link{learnSurvival}}}
\examples{
\dontrun{
	data(beer.exprs)
	set.seed(123)
	X<-t(as.matrix(beer.exprs))
	data(beer.survival)
	y<-Surv(beer.survival[,2],beer.survival[,1])
	
	ls<-generateLearningsets(y=y[,1],method='CV',fold=3,niter=1)

gsel<-geneSelection(X=X,y=y,method='fastCox',LearningSets=ls,criterion='coefficient')
	
tuneres<-tune(X=X,y=y,LearningSets=ls,survmethod='coxBoostSurv',GeneSel=gsel,nbgene=50)



cval <- learnSurvival(X = X, y = y, LearningSets = ls,
			survmethod="coxBoostSurv",
			tuneres=tuneres)



tuneres2<-tune(X=X,y=y,LearningSets=ls,measure='CvPLogL',survmethod='coxBoostSurv',GeneSel=gsel,nbgene=20)

getBestParameters(tuneres,1,measure='HarrellC')

getBestParameters(tuneres2,1)
	
}	

}
\keyword{multivariate}
